'use strict';

var _require = require('fetch-ponyfill')(),
    fetch = _require.fetch;

var _require2 = require('query-string'),
    stringify = _require2.stringify;

var _require3 = require('luxon'),
    DateTime = _require3.DateTime;

var isString = require('lodash/isString');
var isObject = require('lodash/isObject');
var isBoolean = require('lodash/isBoolean');
var groupBy = require('lodash/groupBy');
var sortBy = require('lodash/sortBy');
var toArray = require('lodash/toArray');
var isDate = require('lodash/isDate');
var merge = require('lodash/merge');
var slug = require('slugg');
var getStream = require('get-stream').array;

var _require4 = require('./stations'),
    _stations = _require4.all;

var stations = getStream(_stations());

var defaults = {
	direct: false,
	product: null
};

var products = {
	'KTX': '100',
	'새마을': '101',
	'무궁화': '102', // including 누리로
	'청춘': '104'
};

var hashLeg = function hashLeg(l) {
	return [l.origin.id, l.departure, l.destination.id, l.arrival, l.line.name].join('_');
};
var hashJourney = function hashJourney(j) {
	return j.legs.map(function (l) {
		return l.schedule;
	}).join('_');
};

var createISODate = function createISODate(_ref) {
	var day = _ref.day,
	    time = _ref.time;
	return DateTime.fromFormat(day + ' ' + time, 'yyyyMMdd HHmmss', { zone: 'Asia/Seoul' }).toISO();
};

var createLeg = async function createLeg(l) {
	var stationList = await stations;
	var leg = {
		origin: stationList.find(function (s) {
			return s.id === l.h_dpt_rs_stn_cd;
		}),
		destination: stationList.find(function (s) {
			return s.id === l.h_arv_rs_stn_cd;
		}),
		departure: createISODate({ day: l.h_dpt_dt, time: l.h_dpt_tm }),
		arrival: createISODate({ day: l.h_arv_dt, time: l.h_arv_tm }),
		// todo: h_expct_dlay_hr
		mode: 'train', // sigh…
		public: true,
		operator: 'korail',

		line: {
			type: 'line',
			id: l.h_trn_no + '',
			name: [l.h_trn_clsf_nm, l.h_trn_no].join(' '),
			product: {
				id: l.h_trn_gp_cd,
				name: l.h_trn_gp_nm
			},
			mode: 'train',
			operator: 'korail'
		}
	};

	leg.schedule = slug(hashLeg(leg));
	return leg;
};

var createJourney = async function createJourney(legGroup) {
	if (legGroup.length !== +legGroup[0].h_chg_trn_dv_cd) throw new Error('unexpected leg group length, please report this issue');

	var legs = await Promise.all(legGroup.map(createLeg));

	var journey = {
		type: 'journey',
		id: slug(hashJourney({ legs: legs })),
		legs: legs
	};

	if (legs.length === 1 && legGroup[0].h_rcvd_amt && +legGroup[0].h_rcvd_amt > 0) {
		journey.price = {
			amount: +legGroup[0].h_rcvd_amt,
			currency: 'KRW'
		};
	}

	return journey;
};

var journeys = async function journeys(origin, destination) {
	var date = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : new Date();
	var opt = arguments[3];

	var stationList = await stations;

	if (isString(origin)) origin = stationList.find(function (s) {
		return s.id === origin;
	});else origin = stationList.find(function (s) {
		return s.id === origin.id;
	});
	if (!isObject(origin) || origin.type !== 'station' || !isString(origin.id) || !isString(origin.name)) throw new Error('invalid, missing or unknown origin');
	origin = origin.name;

	if (isString(destination)) destination = stationList.find(function (s) {
		return s.id === destination;
	});else destination = stationList.find(function (s) {
		return s.id === destination.id;
	});
	if (!isObject(destination) || destination.type !== 'station' || !isString(destination.id) || !isString(destination.name)) throw new Error('invalid, missing or unknown destination');
	destination = destination.name;

	if (!isDate(date)) throw new Error('`date` must be a JS Date() object');
	var day = DateTime.fromJSDate(date, { zone: 'Asia/Seoul' }).toFormat('yyyyMMdd');
	var time = DateTime.fromJSDate(date, { zone: 'Asia/Seoul' }).toFormat('HHmmss');

	var options = merge({}, defaults, opt || {});
	if (!isBoolean(options.direct)) throw new Error('invalid options.direct, must be boolean');
	if (options.product && !Object.keys(products).includes(options.product)) throw new Error('invalid options.product, must be one of ' + JSON.stringify(Object.keys(products)));

	var product = options.product ? products[options.product] : '109';

	var endpoint = 'https://smart.letskorail.com/classes/com.korail.mobile.seatMovie.ScheduleView';
	var query = stringify({
		Device: 'IP',
		// Sid: 'GZSeQnD/q53ni/oohPUFlQ==',
		// Version: '180314001',
		ebizCrossCheck: 'Y',
		key: 'korail1234567890',
		radJobId: options.direct ? '1' : '5',
		rtYn: 'N',
		selGoTrain: product,
		srtCheckYn: 'Y',
		txtCardPsgCnt: '0',
		txtGoAbrdDt: day,
		txtGoEnd: destination,
		txtGoHour: time,
		txtGoStart: origin,
		// txtMenuId: '11',
		txtPsgFlg_1: '01', // passenger count, todo
		txtPsgFlg_2: '00',
		txtPsgFlg_3: '00',
		txtPsgFlg_4: '00',
		txtPsgFlg_5: '00',
		txtSeatAttCd_2: '000',
		txtSeatAttCd_3: '000',
		txtSeatAttCd_4: '015',
		txtTrnGpCd: product
	});
	var url = endpoint + '?' + query;
	var results = await fetch(url, {
		method: 'POST',
		headers: {
			'Accept': 'application/json'
		}
	});
	results = await results.json();

	if (results.strResult === 'FAIL' && results.h_msg_cd === 'WRG000000') return []; // no query results error
	if (results.strResult === 'FAIL' && results.h_msg_cd === 'WRD000061') return []; // no direct connections found error
	if (results.strResult !== 'SUCC') throw new Error(JSON.stringify(results));

	var legGroups = toArray(groupBy(results.trn_infos.trn_info, function (x) {
		return x.h_trn_seq;
	})).map(function (lG) {
		return sortBy(lG, function (x) {
			return +x.h_chg_trn_seq;
		});
	});

	return Promise.all(legGroups.map(createJourney));
};

module.exports = journeys;