'use strict';

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var validateArguments = require('fpti-util').validateMethodArguments.journeys;

var _require = require('fetch-ponyfill')(),
    fetch = _require.fetch;

var _require2 = require('query-string'),
    stringify = _require2.stringify;

var _require3 = require('luxon'),
    DateTime = _require3.DateTime;

var merge = require('lodash/merge');
var groupBy = require('lodash/groupBy');
var sortBy = require('lodash/sortBy');
var toArray = require('lodash/toArray');
var last = require('lodash/last');
var take = require('lodash/take');
var slug = require('slugg');
var getStream = require('get-stream').array;

var _require4 = require('./stations'),
    _stations = _require4.all;

var stations = getStream(_stations());

var products = {
	'KTX': '100',
	'새마을': '101',
	'무궁화': '102', // including 누리로
	'청춘': '104'
};

var operator = {
	type: 'operator',
	id: 'korail',
	name: 'korail',
	url: 'https://www.letskorail.com'
};

var hashLeg = function hashLeg(l) {
	return [l.origin.id, l.departure, l.destination.id, l.arrival, l.line.name].join('_');
};
var hashJourney = function hashJourney(j) {
	return j.legs.map(hashLeg).join('_');
};

var createISODate = function createISODate(_ref) {
	var day = _ref.day,
	    time = _ref.time;
	return DateTime.fromFormat(day + ' ' + time, 'yyyyMMdd HHmmss', { zone: 'Asia/Seoul' }).toISO();
};

var createLeg = async function createLeg(l) {
	var stationList = await stations;
	var leg = {
		origin: stationList.find(function (s) {
			return s.id === l.h_dpt_rs_stn_cd;
		}),
		destination: stationList.find(function (s) {
			return s.id === l.h_arv_rs_stn_cd;
		}),
		departure: createISODate({ day: l.h_dpt_dt, time: l.h_dpt_tm }),
		arrival: createISODate({ day: l.h_arv_dt, time: l.h_arv_tm }),
		// todo: h_expct_dlay_hr
		mode: 'train', // sigh…
		public: true,
		operator: operator,

		line: {
			type: 'line',
			id: l.h_trn_no + '',
			name: [l.h_trn_clsf_nm, l.h_trn_no].join(' '),
			product: {
				id: l.h_trn_gp_cd,
				name: l.h_trn_gp_nm
			},
			mode: 'train',
			operator: operator
		}
	};

	leg.tripId = [leg.line.id, DateTime.fromISO(leg.departure, { zone: 'Asia/Seoul' }).toFormat('yyyyMMdd')].join('###');
	return leg;
};

var createJourney = async function createJourney(legGroup) {
	if (legGroup.length !== +legGroup[0].h_chg_trn_dv_cd) throw new Error('unexpected leg group length, please report this issue');

	var legs = await Promise.all(legGroup.map(createLeg));

	var journey = {
		type: 'journey',
		id: slug(hashJourney({ legs: legs })),
		legs: legs
	};

	if (legs.length === 1 && legGroup[0].h_rcvd_amt && +legGroup[0].h_rcvd_amt > 0) {
		journey.price = {
			amount: +legGroup[0].h_rcvd_amt,
			currency: 'KRW'
		};
	}

	return journey;
};

var fetchJourneysForDate = async function fetchJourneysForDate(origin, destination, options, date) {
	var endpoint = 'https://smart.letskorail.com/classes/com.korail.mobile.seatMovie.ScheduleView';
	var day = DateTime.fromJSDate(date, { zone: 'Asia/Seoul' }).toFormat('yyyyMMdd');
	var time = DateTime.fromJSDate(date, { zone: 'Asia/Seoul' }).toFormat('HHmmss');
	var product = options.product ? products[options.product] : '109';
	var query = stringify({
		Device: 'IP',
		// Sid: 'GZSeQnD/q53ni/oohPUFlQ==',
		// Version: '180314001',
		ebizCrossCheck: 'Y',
		key: 'korail1234567890',
		radJobId: options.transfers === 0 ? '1' : '5',
		rtYn: 'N',
		selGoTrain: product,
		srtCheckYn: 'Y',
		txtCardPsgCnt: '0',
		txtGoAbrdDt: day,
		txtGoEnd: destination,
		txtGoHour: time,
		txtGoStart: origin,
		// txtMenuId: '11',
		txtPsgFlg_1: '01', // passenger count, todo
		txtPsgFlg_2: '00',
		txtPsgFlg_3: '00',
		txtPsgFlg_4: '00',
		txtPsgFlg_5: '00',
		txtSeatAttCd_2: '000',
		txtSeatAttCd_3: '000',
		txtSeatAttCd_4: '015',
		txtTrnGpCd: product
	});
	var url = endpoint + '?' + query;
	var results = await fetch(url, {
		method: 'POST',
		headers: {
			'Accept': 'application/json'
		}
	});
	results = await results.json();

	if (results.strResult === 'FAIL' && results.h_msg_cd === 'WRG000000') return []; // no query results error
	if (results.strResult === 'FAIL' && results.h_msg_cd === 'WRD000061') return []; // no direct connections found error
	if (results.strResult !== 'SUCC') throw new Error(JSON.stringify(results));

	var legGroups = toArray(groupBy(results.trn_infos.trn_info, function (x) {
		return x.h_trn_seq;
	})).map(function (lG) {
		return sortBy(lG, function (x) {
			return +x.h_chg_trn_seq;
		});
	});

	return Promise.all(legGroups.map(createJourney));
};

// default options
var defaults = function defaults() {
	return {
		// fpti options
		when: null,
		departureAfter: null,
		results: null,
		transfers: null,
		interval: null,

		// module-specific options
		product: null
	};
};

var journeys = async function journeys(origin, destination) {
	var opt = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

	// merge options with defaults
	var def = defaults();
	if (!(opt.departureAfter || opt.when)) def.departureAfter = new Date();
	var options = merge({}, def, opt);

	// validate arguments, prepare origin and destination
	var stationList = await stations;
	if (typeof origin === 'string') origin = stationList.find(function (s) {
		return s.id === origin;
	});else origin = stationList.find(function (s) {
		return s.id === origin.id;
	});
	if (typeof destination === 'string') destination = stationList.find(function (s) {
		return s.id === destination;
	});else destination = stationList.find(function (s) {
		return s.id === destination.id;
	});
	validateArguments(origin, destination, options);
	if (typeof origin !== 'string') origin = origin.name;
	if (typeof destination !== 'string') destination = destination.name;

	// validate module-specific options
	if (options.product && !Object.keys(products).includes(options.product)) throw new Error('`opt.product` must be one of ' + JSON.stringify(Object.keys(products)));

	var date = options.when || options.departureAfter;
	var endDate = DateTime.fromJSDate(date, { zone: 'Asia/Seoul' }).plus({ minutes: options.interval || 0 }).toJSDate();

	var endOfIntervalReachedOrNoIntervalSet = !options.interval;
	var currentDate = date;
	var journeys = [];
	do {
		var newJourneys = await fetchJourneysForDate(origin, destination, options, currentDate);
		if (newJourneys.length === 0) {
			currentDate = DateTime.fromJSDate(currentDate, { zone: 'Asia/Seoul' }).plus({ minutes: 120 }).toJSDate(); // @todo
		} else {
			var _journeys;

			(_journeys = journeys).push.apply(_journeys, _toConsumableArray(newJourneys));

			var newJourneysSortedByDeparture = sortBy(newJourneys, function (journey) {
				return +new Date(journey.legs[0].departure);
			});
			var latestJourneyDate = new Date(last(newJourneysSortedByDeparture).legs[0].departure);
			var newCurrentDate = DateTime.fromJSDate(latestJourneyDate, { zone: 'Asia/Seoul' }).plus({ minutes: 1 }).toJSDate();
			if (+newCurrentDate === +currentDate) break;
			currentDate = newCurrentDate;
		}
		endOfIntervalReachedOrNoIntervalSet = !options.interval || +currentDate > +endDate;
	} while (!endOfIntervalReachedOrNoIntervalSet);

	journeys = journeys.filter(function (j) {
		return +new Date(j.legs[0].departure) >= +date;
	});
	if (typeof options.interval === 'number') journeys = journeys.filter(function (j) {
		return +new Date(j.legs[0].departure) <= +endDate;
	});
	if (typeof options.transfers === 'number') journeys = journeys.filter(function (j) {
		return j.legs.length <= options.transfers + 1;
	});
	if (typeof options.results === 'number') journeys = take(journeys, options.results);
	return journeys;
};
journeys.features = { // required by fpti
	when: 'Journey date, synonym to departureAfter',
	departureAfter: 'List journeys with a departure (first leg) after this date',
	results: 'Max. number of results returned',
	transfers: 'Max. number of transfers',
	interval: 'Results for how many minutes after / before when (depending on whenRepresents)',
	product: 'Filter for journeys that use the given product'
};

module.exports = journeys;